#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion


namespace NinjaTrader.Indicator
{
    [Description("ADX Breakouts - The ADX signal line goes up whenever volatility increases. When ADX gets over 40, it shows especially strong breakout price action in progress.")]
    public class ADXBreakoutsIndicator : Indicator
    {
        #region Variables
        private int aDXPeriod = 15; // Default setting for ADXPeriod
        private int aDXBO = 40; // Default setting for ADX Breakout Value
		private double pntAbvDlyHi = 0.5;
		private double currentPrice = 0;
		private bool signalValid = true;
		private Color arrowColor = Color.LimeGreen;
		private String aDXBreakoutSignalMessage = @"ADX Breakout";
        #endregion

        
        protected override void Initialize()
        {
            Overlay				= true;
            CalculateOnBarClose = true;
        }
		
        protected override void OnBarUpdate()
        {		
			if (CurrentBar < aDXPeriod) return;
			
			// Condition for ADX Breakout Entries
            if (ADX(ADXPeriod)[0] > ADXBO && signalValid && High[0] >= CurrentDayOHL().CurrentHigh[1] + pntAbvDlyHi)
			{
				DrawArrowDown("Entry Trigger Signal" + Time[0], true, 0, High[0] + 1, arrowColor);
				DrawText("Breakout Entry Text" + ToTime(Time[0]), true, aDXBreakoutSignalMessage, 0, High[0] + 2, 0, Color.Black, new Font("Arial", 8), StringAlignment.Center, Color.Transparent, Color.Transparent, 1);	
				signalValid = false;
			}
			
		    if(!signalValid && ADX(ADXPeriod)[0] < ADXBO )
			{
				signalValid = true;
			}	
		}
		
       #region Properties
        [Description("Value for ADX Period")]
        [GridCategory("Parameters")]
        public int ADXPeriod
        {
            get { return aDXPeriod; }
            set { aDXPeriod = Math.Max(1, value); }
        }

        [Description("ADX Break Out Value")]
        [GridCategory("Parameters")]
        public int ADXBO
        {
            get { return aDXBO; }
            set { aDXBO = Math.Max(1, value); }
        }
		
		[Description("Point value above daily high")]
        [GridCategory("Parameters")]
        public double PntAbvDlyHi
        {
            get { return pntAbvDlyHi; }
            set { pntAbvDlyHi = Math.Max(1, value); }
        }
		
		[Description("ADX Breakout Signal Message")]
        [GridCategory("Parameters")]
        public string ADXBreakoutSignalMessage
        {
            get { return aDXBreakoutSignalMessage; }
            set { aDXBreakoutSignalMessage = value; }
        }
		[XmlIgnore()]
		[Description("Color for Breakout Entry Signal Arrow")]
        [GridCategory("Parameters")]
        public Color ArrowColor
        {
            get { return arrowColor; }
            set { arrowColor = value; }
        }
		
		[Browsable(false)]
		public string arrowColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(arrowColor); }
			set { arrowColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private ADXBreakoutsIndicator[] cacheADXBreakoutsIndicator = null;

        private static ADXBreakoutsIndicator checkADXBreakoutsIndicator = new ADXBreakoutsIndicator();

        /// <summary>
        /// ADX Breakouts - The ADX signal line goes up whenever volatility increases. When ADX gets over 40, it shows especially strong breakout price action in progress.
        /// </summary>
        /// <returns></returns>
        public ADXBreakoutsIndicator ADXBreakoutsIndicator(int aDXBO, string aDXBreakoutSignalMessage, int aDXPeriod, Color arrowColor, double pntAbvDlyHi)
        {
            return ADXBreakoutsIndicator(Input, aDXBO, aDXBreakoutSignalMessage, aDXPeriod, arrowColor, pntAbvDlyHi);
        }

        /// <summary>
        /// ADX Breakouts - The ADX signal line goes up whenever volatility increases. When ADX gets over 40, it shows especially strong breakout price action in progress.
        /// </summary>
        /// <returns></returns>
        public ADXBreakoutsIndicator ADXBreakoutsIndicator(Data.IDataSeries input, int aDXBO, string aDXBreakoutSignalMessage, int aDXPeriod, Color arrowColor, double pntAbvDlyHi)
        {
            if (cacheADXBreakoutsIndicator != null)
                for (int idx = 0; idx < cacheADXBreakoutsIndicator.Length; idx++)
                    if (cacheADXBreakoutsIndicator[idx].ADXBO == aDXBO && cacheADXBreakoutsIndicator[idx].ADXBreakoutSignalMessage == aDXBreakoutSignalMessage && cacheADXBreakoutsIndicator[idx].ADXPeriod == aDXPeriod && cacheADXBreakoutsIndicator[idx].ArrowColor == arrowColor && Math.Abs(cacheADXBreakoutsIndicator[idx].PntAbvDlyHi - pntAbvDlyHi) <= double.Epsilon && cacheADXBreakoutsIndicator[idx].EqualsInput(input))
                        return cacheADXBreakoutsIndicator[idx];

            lock (checkADXBreakoutsIndicator)
            {
                checkADXBreakoutsIndicator.ADXBO = aDXBO;
                aDXBO = checkADXBreakoutsIndicator.ADXBO;
                checkADXBreakoutsIndicator.ADXBreakoutSignalMessage = aDXBreakoutSignalMessage;
                aDXBreakoutSignalMessage = checkADXBreakoutsIndicator.ADXBreakoutSignalMessage;
                checkADXBreakoutsIndicator.ADXPeriod = aDXPeriod;
                aDXPeriod = checkADXBreakoutsIndicator.ADXPeriod;
                checkADXBreakoutsIndicator.ArrowColor = arrowColor;
                arrowColor = checkADXBreakoutsIndicator.ArrowColor;
                checkADXBreakoutsIndicator.PntAbvDlyHi = pntAbvDlyHi;
                pntAbvDlyHi = checkADXBreakoutsIndicator.PntAbvDlyHi;

                if (cacheADXBreakoutsIndicator != null)
                    for (int idx = 0; idx < cacheADXBreakoutsIndicator.Length; idx++)
                        if (cacheADXBreakoutsIndicator[idx].ADXBO == aDXBO && cacheADXBreakoutsIndicator[idx].ADXBreakoutSignalMessage == aDXBreakoutSignalMessage && cacheADXBreakoutsIndicator[idx].ADXPeriod == aDXPeriod && cacheADXBreakoutsIndicator[idx].ArrowColor == arrowColor && Math.Abs(cacheADXBreakoutsIndicator[idx].PntAbvDlyHi - pntAbvDlyHi) <= double.Epsilon && cacheADXBreakoutsIndicator[idx].EqualsInput(input))
                            return cacheADXBreakoutsIndicator[idx];

                ADXBreakoutsIndicator indicator = new ADXBreakoutsIndicator();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.ADXBO = aDXBO;
                indicator.ADXBreakoutSignalMessage = aDXBreakoutSignalMessage;
                indicator.ADXPeriod = aDXPeriod;
                indicator.ArrowColor = arrowColor;
                indicator.PntAbvDlyHi = pntAbvDlyHi;
                Indicators.Add(indicator);
                indicator.SetUp();

                ADXBreakoutsIndicator[] tmp = new ADXBreakoutsIndicator[cacheADXBreakoutsIndicator == null ? 1 : cacheADXBreakoutsIndicator.Length + 1];
                if (cacheADXBreakoutsIndicator != null)
                    cacheADXBreakoutsIndicator.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheADXBreakoutsIndicator = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// ADX Breakouts - The ADX signal line goes up whenever volatility increases. When ADX gets over 40, it shows especially strong breakout price action in progress.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ADXBreakoutsIndicator ADXBreakoutsIndicator(int aDXBO, string aDXBreakoutSignalMessage, int aDXPeriod, Color arrowColor, double pntAbvDlyHi)
        {
            return _indicator.ADXBreakoutsIndicator(Input, aDXBO, aDXBreakoutSignalMessage, aDXPeriod, arrowColor, pntAbvDlyHi);
        }

        /// <summary>
        /// ADX Breakouts - The ADX signal line goes up whenever volatility increases. When ADX gets over 40, it shows especially strong breakout price action in progress.
        /// </summary>
        /// <returns></returns>
        public Indicator.ADXBreakoutsIndicator ADXBreakoutsIndicator(Data.IDataSeries input, int aDXBO, string aDXBreakoutSignalMessage, int aDXPeriod, Color arrowColor, double pntAbvDlyHi)
        {
            return _indicator.ADXBreakoutsIndicator(input, aDXBO, aDXBreakoutSignalMessage, aDXPeriod, arrowColor, pntAbvDlyHi);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// ADX Breakouts - The ADX signal line goes up whenever volatility increases. When ADX gets over 40, it shows especially strong breakout price action in progress.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ADXBreakoutsIndicator ADXBreakoutsIndicator(int aDXBO, string aDXBreakoutSignalMessage, int aDXPeriod, Color arrowColor, double pntAbvDlyHi)
        {
            return _indicator.ADXBreakoutsIndicator(Input, aDXBO, aDXBreakoutSignalMessage, aDXPeriod, arrowColor, pntAbvDlyHi);
        }

        /// <summary>
        /// ADX Breakouts - The ADX signal line goes up whenever volatility increases. When ADX gets over 40, it shows especially strong breakout price action in progress.
        /// </summary>
        /// <returns></returns>
        public Indicator.ADXBreakoutsIndicator ADXBreakoutsIndicator(Data.IDataSeries input, int aDXBO, string aDXBreakoutSignalMessage, int aDXPeriod, Color arrowColor, double pntAbvDlyHi)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.ADXBreakoutsIndicator(input, aDXBO, aDXBreakoutSignalMessage, aDXPeriod, arrowColor, pntAbvDlyHi);
        }
    }
}
#endregion
